package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysDicMapper
import com.gxzc.zen.api.sys.model.SysDic
import com.gxzc.zen.api.sys.service.ISysDicService
import com.gxzc.zen.common.contants.CACHEKEYS
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.util.CacheUtil
import com.gxzc.zen.orm.annotation.ZenTransactional
import org.slf4j.LoggerFactory
import org.springframework.boot.CommandLineRunner
import org.springframework.stereotype.Service

/**
 * <p>
 * 字典表 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Suppress("UNCHECKED_CAST")
@Service
class SysDicServiceImpl : ServiceImpl<SysDicMapper, SysDic>(), ISysDicService, CommandLineRunner {
    companion object {
        private val logger = LoggerFactory.getLogger(SysDicServiceImpl::class.java)
        const val CACHE_KEY_ALL = "dic_all"
    }

    override fun run(vararg args: String?) {
        logger.debug("${this::class.simpleName} init.")
        getListCacheable()
    }

    override fun getListCacheable(): MutableList<SysDic> {
        val cached = CacheUtil.get(CACHEKEYS.SYS, CACHE_KEY_ALL)?.get() as? MutableList<SysDic>
        if (cached != null) {
            return cached
        }
        return baseMapper.selectByParams(null)
    }

    override fun getList(enable: Boolean?): MutableList<SysDic> {
        return if (enable != null) {
            baseMapper.selectByParams(mutableMapOf(
                    "enable" to enable
            ))
        } else {
            baseMapper.selectByParams(null)
        }
    }

    override fun getListByKey(key: String): MutableList<SysDic> {
        val allData = getListCacheable()
        return allData.filter {
            it.enable != null && it.enable!! && it.key == key
        }.toMutableList()
    }

    @ZenTransactional
    override fun modify(data: SysDic): SysDic {
        baseMapper.updateNoLogic(data, EntityWrapper<SysDic>().eq("id", data.id))
        // 更新缓存
        val cached = CacheUtil.get(CACHEKEYS.SYS, CACHE_KEY_ALL)?.get() as? MutableList<SysDic>
        cached?.let {
            val idx = it.indexOfFirst { it.id == data.id }
            if (idx != -1) {
                it[idx] = data
            }
        }
        return data
    }

    override fun getOneByKey(key: String, value: String?, sort: Int?): SysDic? {
        val result = baseMapper.selectByParams(mutableMapOf(
                "key" to key,
                "value" to value!!,
                "sort" to sort!!,
                "enable" to true
        ))
        return if (result.size > 0) {
            result.filter {
                it.sort == sort && it.value == value
            }[0]
        } else {
            null
        }
    }

    @ZenTransactional
    override fun insertCacheable(data: SysDic) {
        if (baseMapper.insert(data) == 0) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
        val cached = CacheUtil.get(CACHEKEYS.SYS, CACHE_KEY_ALL)?.get() as? MutableList<SysDic>
        cached?.add(data)
    }

    @ZenTransactional
    override fun physicalDeleteCacheable(id: Long) {
        if (baseMapper.physicalDelete(EntityWrapper<SysDic>().eq("id", id)) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
        }
        //
        val cached = CacheUtil.get(CACHEKEYS.SYS, CACHE_KEY_ALL)?.get() as? MutableList<SysDic>
        cached?.let {
            it.removeIf {
                it.id == id
            }
        }
    }
}
