package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysUserMapper
import com.gxzc.zen.api.sys.model.SysUser
import com.gxzc.zen.api.sys.service.ISysUserService
import com.gxzc.zen.common.contants.CACHEKEYS
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.util.CacheUtil
import com.gxzc.zen.orm.annotation.ZenTransactional
import org.slf4j.LoggerFactory
import org.springframework.boot.CommandLineRunner
import org.springframework.stereotype.Service

/**
 * <p>
 * 用户管理 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Suppress("UNCHECKED_CAST")
@Service
class SysUserServiceImpl : ServiceImpl<SysUserMapper, SysUser>(), ISysUserService, CommandLineRunner {
    companion object {
        private val logger = LoggerFactory.getLogger(SysUserServiceImpl::class.java)
        const val CACHE_KEY_ALL = "all"
    }

    override fun run(vararg args: String?) {
        logger.debug("${this::class.simpleName} init.")
        getListCacheable()
    }

    override fun getListCacheable(): MutableList<SysUser> {
        val cached = CacheUtil.get(CACHEKEYS.USER, CACHE_KEY_ALL)?.get() as? MutableList<SysUser>
        if (cached != null) {
            return cached
        }
        val ret = baseMapper.selectByParams(null)
        CacheUtil.put(CACHEKEYS.USER, CACHE_KEY_ALL, ret)
        return ret
    }

    override fun getUserByAccountCacheable(account: String): SysUser? {
        return getListCacheable().find {
            it.account == account
        }
    }

    override fun getUserByIdCacheable(id: Long): SysUser? {
        return getListCacheable().find {
            it.id == id
        }
    }

    @ZenTransactional
    override fun insertCacheable(entity: SysUser) {
        if (baseMapper.insert(entity) == 0) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
        val cached = CacheUtil.get(CACHEKEYS.USER, CACHE_KEY_ALL)?.get() as? MutableList<SysUser>
        cached?.add(entity)
    }

    @ZenTransactional
    override fun modify(entity: SysUser): SysUser {
        baseMapper.updateNoLogic(entity, EntityWrapper<SysUser>().eq("id", entity.id))
        // 更新缓存
        val cached = CacheUtil.get(CACHEKEYS.USER, CACHE_KEY_ALL)?.get() as? MutableList<SysUser>
        cached?.let {
            val idx = it.indexOfFirst { it.id == entity.id }
            if (idx != -1) {
                it[idx] = entity
            }
        }
        return entity
    }

    @ZenTransactional
    override fun physicalDeleteCacheable(id: Long) {
        if (baseMapper.physicalDelete(EntityWrapper<SysUser>().eq("id", id)) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
        }
        //
        val cached = CacheUtil.get(CACHEKEYS.USER, CACHE_KEY_ALL)?.get() as? MutableList<SysUser>
        cached?.let {
            it.removeIf {
                it.id == id
            }
        }
    }
}
