package com.gxzc.zen.web.sys.controller

import com.gxzc.zen.api.sys.service.ISysRoleService
import com.gxzc.zen.common.base.BaseController
import com.gxzc.zen.common.dto.ResponseDto
import com.gxzc.zen.common.util.PaginationUtil
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

/**
 * 角色 控制器
 * @author NorthLan
 * @date 2018/3/24
 * @url https://noahlan.com
 */
@RestController
@RequestMapping("role")
class RoleController : BaseController() {
    companion object {
        private val logger = LoggerFactory.getLogger(RoleController::class.java)
    }

    @Autowired
    private lateinit var roleService: ISysRoleService

    @GetMapping("list")
    fun list(@RequestParam(required = false) keyword: String?,
             @RequestParam(required = false) searchOption: Int?): ResponseEntity<*> {
        var result: Any? = null
        if (!keyword.isNullOrEmpty() && searchOption != null) {
            when (searchOption) {
                1 -> {
                    result = if (PaginationUtil.paginable(getRequest())) {
                        roleService.getListByParamPage(keyword, null, null, PaginationUtil.getCurrent(getRequest())!!, PaginationUtil.getPageSize(getRequest())!!)
                    } else {
                        roleService.getListByParam(keyword, null, null)
                    }
                }
                2 -> {
                    result = if (PaginationUtil.paginable(getRequest())) {
                        roleService.getListByParamPage(null, keyword, null, PaginationUtil.getCurrent(getRequest())!!, PaginationUtil.getPageSize(getRequest())!!)
                    } else {
                        roleService.getListByParam(null, keyword, null)
                    }
                }
                else -> {
                    // 未知的查询关键字类型
                    logger.warn("Unknown searchOption: [$searchOption]")
                }
            }
        } else {
            result = if (PaginationUtil.paginable(getRequest())) {
                roleService.getListByParamPage(null, null, null, PaginationUtil.getCurrent(getRequest())!!, PaginationUtil.getPageSize(getRequest())!!)
            } else {
                roleService.getListByParam(null, null, null)
            }

        }
        return ResponseEntity.ok(ResponseDto().apply { data = result })
    }
}